package 
{
	import flash.filters.BitmapFilter;
	import flash.filters.BitmapFilterQuality;
	import flash.filters.BlurFilter;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.World;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import flash.geom.Point;
	import net.flashpunk.utils.Draw;
	/**
	 * ...
	 * @author Time
	 */
	public class Game extends World
	{
		private var filter:BitmapFilter;
		private static const minBlur:int = 1;
		private static const maxBlur:int = 6;
		
		private static var blurCycle:Boolean = false;
		private static var blurKey:int = Key.B;
		
		public static var controls:Text;
		public static var controlsHidden:Boolean = false;
		private static var controlsKey:int = Key.H;
		
		private static const maxT:int = 180;
		private static var t:int = 0;
		
		public static const particles:int = 2000;
		public static const penSize:int = 30;
		private static var drawGrid:Boolean = false;
		private static var drawField:Boolean = false;
		private static const drawGridKey:int = Key.G;
		private static const drawFieldKey:int = Key.F;
		
		public static var vectors:Vector.<Vector.<Point>> = new Vector.<Vector.<Point>>();
		public static const gridWidth:int = 4;
		public static const gridHeight:int = 4;
		public static const gridX:int = FP.width / gridWidth; //The number of grids width-wise
		public static const gridY:int = FP.height / gridHeight; //The number of grids height-wise
		
		private var lastMouse:Point = new Point();
		
		public function Game() 
		{
			//FP.console.enable();
			super();
		}
		override public function begin():void
		{
			super.begin();
			
			for (var i:int = 0; i < gridX; i++)
			{
				vectors.push(new Vector.<Point>());
				for (var j:int = 0; j < gridY; j++)
				{
					var _x:int = i * gridWidth + gridWidth / 2;
					var _y:int = j * gridHeight + gridHeight / 2;
					vectors[i].push(new Point());
				}
			}
			
			for (i = 0; i < particles; i++)
			{
				add(new Particle(FP.width * Math.random(), FP.height * Math.random()));
			}
			
			controls = new Text("H: Toggle Help\nF: Toggle Field\nG: Toggle Grid\nB: Toggle Blur Cycle");
		}
		override public function update():void
		{
			var _x:int = getX(Input.mouseX);
			var _y:int = getY(Input.mouseY);
			for (var i:int = _x - penSize/gridWidth; i < _x + penSize/gridWidth; i++)
			{
				for (var j:int = _y - penSize/gridHeight; j < _y + penSize/gridHeight; j++)
				{
					if (Input.mouseDown && FP.distance(Input.mouseX, Input.mouseY, lastMouse.x, lastMouse.y) > 1 && FP.distance((i + 1 / 2) * gridWidth, (j + 1 / 2) * gridHeight, Input.mouseX, Input.mouseY) <= penSize)
					{
						vectors[(i+gridX)%gridX][(j+gridY)%gridY] = new Point(Input.mouseX - lastMouse.x, Input.mouseY - lastMouse.y);
						vectors[(i+gridX)%gridX][(j+gridY)%gridY].normalize(Math.min(vectors[(i+gridX)%gridX][(j+gridY)%gridY].length, 10));
					}
				}
			}
			super.update();
			lastMouse = new Point(Input.mouseX, Input.mouseY);
			if (blurCycle)
			{
				t = (t + 1) % maxT;
			}
			
			if (Input.released(drawGridKey))
			{
				drawGrid = !drawGrid;
			}
			if (Input.released(drawFieldKey))
			{
				drawField = !drawField;
			}
			if (Input.released(blurKey))
			{
				blurCycle = !blurCycle;
			}
			if (Input.released(controlsKey))
			{
				controlsHidden = !controlsHidden;
			}
		}
		override public function render():void
		{
			super.render();
			
			var lerp:Number = FP.lerp(minBlur, maxBlur, (-Math.cos(Math.PI * 2 / maxT * t)+1)/2);
			filter = new BlurFilter(lerp, lerp, BitmapFilterQuality.HIGH);
			FP.buffer.applyFilter(FP.buffer, FP.buffer.rect, new Point(), filter);
			
			if (drawGrid)
			{
				for (var i:int = 0; i < gridX; i++)
				{
					Draw.linePlus(i * gridWidth, 0, i * gridWidth, FP.height, 0x00FF00, 0.1);
				}
				for (var j:int = 0; j < gridY; j++)
				{
					Draw.linePlus(0, j * gridHeight, FP.width, j * gridHeight, 0x00FF00, 0.1);
				}
			}
			if (drawField)
			{
				for (i = 0; i < gridX; i++)
				{
					for (j = 0; j < gridY; j++)
					{
						Draw.line(i * gridWidth + gridWidth / 2, j * gridHeight + gridHeight / 2, i * gridWidth + gridWidth / 2 + vectors[i][j].x, j * gridHeight + gridHeight / 2 + vectors[i][j].y, 0x00FF00);
					}
				}
			}
			
			if (!controlsHidden)
			{
				controls.color = 0x000000;
				controls.render(new Point(FP.camera.x + 1, FP.camera.y + FP.height - controls.height - 3), FP.camera);
				controls.render(new Point(FP.camera.x + 1, FP.camera.y + FP.height - controls.height - 1), FP.camera);
				controls.render(new Point(FP.camera.x + 3, FP.camera.y + FP.height - controls.height - 3), FP.camera);
				controls.render(new Point(FP.camera.x + 3, FP.camera.y + FP.height - controls.height - 1), FP.camera);
				
				controls.color = 0xFFFFFF;
				controls.render(new Point(FP.camera.x + 2, FP.camera.y + FP.height - controls.height - 2), FP.camera);
			}
		}
		
		public static function getX(x:int):int
		{
			return Math.floor((x + FP.width) % FP.width / Game.gridWidth);
		}
		public static function getY(y:int):int
		{
			return Math.floor((y + FP.height) % FP.height / Game.gridHeight);
		}
		
	}

}